from pathlib import Path
import filetype
import typer

from ffprobe import FFProbe


def get_codec(path: str):
    try:
        metadata = FFProbe(path)
    except:
        return None

    if 'video' in metadata.streams:
        return metadata.video[0].codec_name

    return metadata.audio[0].codec_name


def convertion_path(path: Path, audio: bool ):

    if not audio:

        if path.suffix.lower() not in ['.mkv', '.mp4']:

            return path.parent / (path.stem + '.mkv')

    else:

        if path.suffix.lower() not in ['.aac', '.m4a']:

            return path.parent / (path.stem + '.aac')

    return path.parent / ('conv-' + path.name)



def check_if_video(path: str):

    guess = filetype.guess(path)

    return guess and 'video' in guess

def check_ignore(file_path, ignore_extension:str, ignore_path:str):

    ignored_by_extension = ignore_extension is not None \
        and str(file_path).lower().endswith(ignore_extension)
    ignored_by_path = ignore_path is not None \
        and str(ignore_path) in str(file_path)

    if ignored_by_extension or ignored_by_path:
        typer.secho(f'Ignoring: {file_path}')
        return True

    return False



import sys
import re
import os
import ffmpeg
import enlighten
if sys.platform == 'win32':
    import wexpect as expect
    # patch windows consloe for scale correctly characters
    import ansicon
    ansicon.load()
else:
    import pexpect as expect

__version__ = '0.1.8'

pattern_duration = re.compile('duration[ \t\r]?:[ \t\r]?(.+?),[ \t\r]?start',re.IGNORECASE)
pattern_progress = re.compile('time=(.+?)[ \t\r]?bitrate',re.IGNORECASE)
BAR_FMT = u'{desc}{desc_pad}{percentage:3.0f}%|{bar}| {count:{len_total}.1f}/{total:.1f} ' + \
          u'[{elapsed}<{eta}, {rate:.2f}{unit_pad}{unit}/s]'

COUNTER_FMT = u'{desc}{desc_pad}{count:.1f} {unit}{unit_pad}' + \
              u'[{elapsed}, {rate:.2f}{unit_pad}{unit}/s]{fill}'

def convert_file(source: str, dest: str):
    stream = ffmpeg.input(source)
    stream = ffmpeg.output(stream, dest, vcodec='libx265', crf='28')
    ffmpeg.run(stream)

def convert_video_progress_bar(source: str, dest: str, manager=None):
    if manager is None:
        manager = enlighten.get_manager()
    stream = ffmpeg.input(source)
    stream = ffmpeg.output(stream, dest, vcodec='libx265', crf='28')
    args = ffmpeg.compile(stream, 'ffmpeg')
    args.insert(1,'-progress pipe:1')
    args = map(lambda x: '"'+x+'"' if '\\' in x or '/' in x else x,args)
    args = list(args)
    name = source.rsplit(os.path.sep,1)[-1]
    proc = expect.spawn(' '.join(args), encoding='utf-8')
    pbar = None
    try:
        proc.expect(pattern_duration)
        total = sum(map(lambda x: float(x[1])*60**x[0],enumerate(reversed(proc.match.groups()[0].strip().split(':')))))
        cont = 0
        pbar = manager.counter(total=100, desc=name, unit='%',bar_format=BAR_FMT, counter_format=COUNTER_FMT)
        while True:
            proc.expect(pattern_progress)
            progress = sum(map(lambda x: float(x[1])*60**x[0],enumerate(reversed(proc.match.groups()[0].strip().split(':')))))
            percent = progress/total*100
            pbar.update(percent-cont)
            cont = percent
    except expect.EOF:
        pass
    finally:
        if pbar is not None:
            pbar.close()
    proc.expect(expect.EOF)
    res = proc.before
    res += proc.read()
    exitstatus = proc.wait()
    if exitstatus:
        raise ffmpeg.Error('ffmpeg','',res)



import typer
from pathlib import Path
import filetype
import os
import shutil
from typer.colors import RED, GREEN
import enlighten
import ffmpeg
from .utils import convertion_path, get_codec, check_ignore
from . import convert_file, convert_video_progress_bar

app = typer.Typer()


@app.callback()
def callback():
    """
    Awesome Portal Gun
    """


@app.command()
def folder(path: Path = typer.Argument(
    default='.',
    exists=True,
    file_okay=True,
    dir_okay=True,
    readable=True,
    resolve_path=True
), ignore_extension: str = typer.Option(
    default=None
), ignore_path: Path = typer.Option(
    default=None,
    exists=True,
    file_okay=True,
    dir_okay=True,
    readable=True,
    resolve_path=True
)):
    """
    Convert all videos and audios in a folder
    """

    videos = []
    audios = []

    for dir, folders, files in os.walk(path):
        base_dir = Path(dir)
        for item in files:

            file_path = base_dir / item
            guess = filetype.guess(str(file_path))

            if check_ignore(file_path, ignore_extension, ignore_path):
                continue

            if guess and 'video' in guess.mime :

                videos.append(file_path)

            if guess and 'audio' in guess.mime:

                audios.append(file_path)

    manager = enlighten.get_manager()
    errors_files = []
    pbar = manager.counter(total=len(videos)+len(audios), desc='Files', unit='files')

    for video in videos:
        typer.secho(f'Processing: {video}')
        if get_codec(str(video)) != 'hevc':
            new_path = convertion_path(video, False)

            try:
                #convert_video(str(video), str(new_path))
                convert_video_progress_bar(str(video), str(new_path), manager)
                os.remove(str(video))
                if video.suffix == new_path.suffix:
                    shutil.move(new_path, str(video))

            except ffmpeg._run.Error:
                typer.secho(f'ffmpeg could not process: {str(video)}', fg=RED)
                errors_files.append(video)

        pbar.update()

    for audio in audios:
        typer.secho(f'Processing: {audio}')
        if get_codec(str(audio)) != 'hevc':

            new_path = convertion_path(audio, True)

            try:

                convert_file(str(audio),str(new_path))

                os.remove(str(audio))
                if audio.suffix == new_path.suffix:
                    shutil.move(new_path, str(audio))

            except ffmpeg._run.Error:
                typer.secho(f'ffmpeg could not process this file: {str(audio)}', fg=RED)
                errors_files.append(audio)


        pbar.update()

    if errors_files:
        typer.secho('This videos could not be processed:', fg=RED)
        typer.secho(str(errors_files), fg=RED)


@app.command()
def file(path: Path = typer.Argument(
    default=None,
    exists=True,
    file_okay=True,
    dir_okay=False,
    readable=True,
    resolve_path=True
)):
    """
    Convert a file
    """

    if path is None:
        typer.secho('Please write the video or audio path', fg=RED)
        return

    guess = filetype.guess(str(path))

    if guess and 'video' in guess.mime:
        conv_path = convertion_path(path, False)
    else:
        conv_path = convertion_path(path, False)

    if conv_path.exists():
        typer.secho('The destination file already exist, \
                    please delete it', fg=RED)
        return


    if get_codec(str(path)) == 'hevc':
        typer.secho('This video codec is already \'hevc\'', fg=GREEN)
        return

    try:
        convert_video_progress_bar(str(path), str(conv_path))
        #convert_video(str(path), str(conv_path))

    except FileNotFoundError as error:
        if error.filename == 'ffmpeg':
            readme_url = 'https://github.com/hiancdtrsnm/video-diet#FFMPEG'
            typer.secho('It seems you don\'t have ffmpeg installed', fg=RED)
            typer.secho(f'Check FFMPEG secction on {readme_url}', fg=RED)
        else:
            raise error
